@@ -6,7 +6,6 @@ require(NNS)
 require(knitr)
 require(rgl)
 require(data.table)
-require(plyr)
 
 ## ----linear,results='hide'-----------------------------------------------
 x = seq(-5, 5, .05); y = x ^ 3
@@ -26,10 +25,10 @@ NNS.reg(x, y, order = 4, noise.reduction = "off")
 f= function(x, y) x ^ 3 + 3 * y - y ^ 3 - 3 * x
 y = x ; z = expand.grid(x, y)
 g = f(z[ , 1], z[ , 2])
-NNS.reg(z, g, order = "max")
+NNS.reg(z, g, order = "max", ncores = 1)
 
 ## ----iris point.est,fig.width=5,fig.height=3,fig.align = "center"--------
-NNS.reg(iris[ , 1 : 4], iris[ , 5], point.est = iris[1 : 10, 1 : 4], type = "CLASS", location = "topleft")$Point.est
+NNS.reg(iris[ , 1 : 4], iris[ , 5], type = "CLASS", point.est = iris[141:150, 1 : 4], location = "topleft")$Point.est
 
 ## ----nonlinear class,fig.width=5,fig.height=3,fig.align = "center"-------
 NNS.reg(iris[ , 1 : 4], iris[ , 5], dim.red.method = "cor", location = "topleft")$equation
